using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace CIS297_FinalProj
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>

    enum GameState
    {
        Start,
        Help
    }
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        private SpriteFont gameFont;

        //Variables
        Texture2D cirTexture;
        Texture2D platTexture;
        Texture2D mainTexture;
        Texture2D animBgTexture;
        Vector2 cirPos;                         //Position of the circle
        Vector2 origin;                         //Center of the circle
        KeyboardState state;                    //Used to determine key presses
        const float cirSpeed = 10.0f;            //Speed of the circle
        const int regPt = 25;                   //Regular points value
        const int spcPt = 100;                  //Special points value
        public int highscore = 0;               //Holds number of points
        private float timeSoFar = 0.0f;         //The total running time
        bool pastMenu = false;

        //Audio
        AudioEngine audioEngine;
        SoundBank soundBank;
        WaveBank waveBank;
        Cue EngineSound = null;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            //Size of screen
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load in the assets
            cirTexture = Content.Load<Texture2D>(@"images\circle");
            platTexture = Content.Load<Texture2D>(@"images\platform");
            mainTexture = Content.Load<Texture2D>(@"images\main");
            // add animated background here
            gameFont = Content.Load<SpriteFont>(@"Font\\SpriteFont1");
            audioEngine = new AudioEngine(@"Content\Audio\audio.xgs");
            soundBank = new SoundBank(audioEngine, @"Content\Audio\Sound Bank.xsb");
            waveBank = new WaveBank(audioEngine, @"Content\Audio\Wave Bank.xwb");

            // Find the center of the circle
            origin.X = cirTexture.Width / 2;
            origin.Y = cirTexture.Height / 2;
            // Set position of the circle
            cirPos = new Vector2(Window.ClientBounds.Width / 4,
                        Window.ClientBounds.Height);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            GamePadState padState = GamePad.GetState(PlayerIndex.One);      //Load up first player to controller

            // Allows the game to exit
            if ((GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed ||
                Keyboard.GetState().IsKeyDown(Keys.Escape)))
                this.Exit();

            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;   //The amount of time since last called

            timeSoFar += (float)gameTime.ElapsedGameTime.TotalSeconds;      //Continually update amount of time passed
            
            pastMenu = true;                    //Disable moving up and down

            // Update status of keys pressed
            state = Keyboard.GetState();

            //** Keyboard Support **//
            if (pastMenu != true)
            {
                //Move Up
                if (state.IsKeyDown(Keys.Up))
                    cirPos.Y -= cirSpeed;
                //Move Down
                if (state.IsKeyDown(Keys.Down))
                    cirPos.Y += cirSpeed;
            }
            //Move Left
            if (state.IsKeyDown(Keys.Left))
                cirPos.X -= cirSpeed;
            //Move Right
            if (state.IsKeyDown(Keys.Right))
                cirPos.X += cirSpeed;

            //** GamePad support ** //
            cirPos.X += padState.ThumbSticks.Left.X * cirSpeed;
            cirPos.Y -= padState.ThumbSticks.Left.Y * cirSpeed;

            CheckBounds();          //Make sure object is within viewable area

            base.Update(gameTime);
        }

        private void CheckBounds()
        {
            //Checks to make sure that the ship is in bounds of the window
            // if it is past bound, set position to be at boundary

            //Prevent ship from going offscreen to the left
            if (cirPos.X < 0 + cirTexture.Width / 2)
                cirPos.X = cirTexture.Width / 2;
            //Prevent ship from going offscreen to the right
            if (cirPos.X > Window.ClientBounds.Width / 2 /*+ cirTexture.Width / 2*/)
                cirPos.X = Window.ClientBounds.Width / 2 /*+ cirTexture.Width / 2*/;
            //Prevent ship from going offscreen too low
            if (cirPos.Y < cirTexture.Height / 2)
                cirPos.Y = cirTexture.Height / 2;
            //Preven ship from going offscreen too high
            if (cirPos.Y > Window.ClientBounds.Height - cirTexture.Height / 2)
                cirPos.Y = Window.ClientBounds.Height - cirTexture.Height / 2;
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //Draw the stuff

            spriteBatch.Begin();
                //Draw the background
                spriteBatch.Draw(mainTexture,
                    new Vector2(0, 0), null, Color.LimeGreen);
                

                //Draw the platforms
                // End draw platforms

                //Draw the circle
                spriteBatch.Draw(cirTexture, cirPos,
                            null, Color.White, 0, origin, 1, SpriteEffects.None, 1);

                //Draw the scoreboard
                spriteBatch.DrawString(gameFont, "High Score: " + highscore.ToString(),
                    new Vector2((2*Window.ClientBounds.Width/3), (Window.ClientBounds.Height/3)),
                        Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
