using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace CIS297_FinalProj
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>

    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        private SpriteFont gameFont;

        //Variables
        Texture2D cirTexture;
        Texture2D platTexture;
        Texture2D mainTexture;
        Texture2D animBgTexture;
        Texture2D menuTexture;
        Texture2D menu2Texture;
        Vector2 cirPos;                         //Position of the circle
        Vector2 origin;                         //Center of the circle
        Rectangle cirBound;
        KeyboardState oldstate;                 //Used to determine key press and release
        KeyboardState state;                    //Used to determine key presses
        const float cirSpeed = 10.0f;            //Speed of the circle
        const float gravity = 5.0f;            //Set gravity
        const int regPt = 25;                   //Regular points value
        const int spcPt = 100;                  //Special points value
        public int highscore = 0;               //Holds number of points
        private float timeSoFar = 0.0f;         //The total running time
        bool pastMenu = false;
        bool instMenu = false;
        bool jumped = false;

        //Audio
        AudioEngine audioEngine;
        SoundBank soundBank;
        WaveBank waveBank;
        Cue EngineSound = null;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            //Size of screen
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Services.AddService(typeof(SpriteBatch), spriteBatch);

            // Load in the assets
            cirTexture = Content.Load<Texture2D>(@"images\circle");
            platTexture = Content.Load<Texture2D>(@"images\platform");
            mainTexture = Content.Load<Texture2D>(@"images\main");
            menuTexture = Content.Load<Texture2D>(@"images\Menu");
            menu2Texture = Content.Load<Texture2D>(@"images\Instructions");
            // add animated background here
            gameFont = Content.Load<SpriteFont>(@"Font\\SpriteFont1");
            audioEngine = new AudioEngine(@"Content\Audio\audio.xgs");
            soundBank = new SoundBank(audioEngine, @"Content\Audio\Sound Bank.xsb");
            waveBank = new WaveBank(audioEngine, @"Content\Audio\Wave Bank.xwb");

            // Find the center of the circle
            origin.X = cirTexture.Width / 2;
            origin.Y = cirTexture.Height / 2;
            // Set position of the circle
            cirPos = new Vector2(Window.ClientBounds.Width / 4,
                        Window.ClientBounds.Height);
            
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            GamePadState padState = GamePad.GetState(PlayerIndex.One);      //Load up first player to controller

            // Allows the game to exit
            if ((GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed ||
                Keyboard.GetState().IsKeyDown(Keys.Escape)))
                this.Exit();

            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;   //The amount of time since last called

            timeSoFar += (float)gameTime.ElapsedGameTime.TotalSeconds;      //Continually update amount of time passed
            
            //pastMenu = true;                    //Disable moving up and down

            // Update status of keys pressed
            state = Keyboard.GetState();

            //If pressed the Enter or Start key, begin the game
            if (padState.Buttons.Start == ButtonState.Pressed ||
                state.IsKeyDown(Keys.Enter))
            {
                if (state.IsKeyDown(Keys.Enter) && !oldstate.IsKeyDown(Keys.Enter))
                {
                    pastMenu = true;
                }
                if (padState.IsButtonDown(Buttons.Start))
                {
                    pastMenu = true;
                }
            }

            //If pressed the Space or Select key, show instructions
            if (padState.Buttons.Back == ButtonState.Pressed ||
                state.IsKeyDown(Keys.Space))
            {
                if (state.IsKeyDown(Keys.Space) && !oldstate.IsKeyDown(Keys.Space))
                {
                    instMenu = true;
                }
                if (padState.IsButtonDown(Buttons.Back))
                {
                    instMenu = true;
                }
            }
            //Press B to exit instructions
            if (padState.IsButtonDown(Buttons.B) ||
                state.IsKeyDown(Keys.B))
                instMenu = false;

            if (pastMenu == true)
            {
                //** Keyboard Support **//

                //Move Left
                if (state.IsKeyDown(Keys.Left))
                    cirPos.X -= cirSpeed;
                //Move Right
                if (state.IsKeyDown(Keys.Right))
                    cirPos.X += cirSpeed;

                //Allow player to jump once
                if (state.IsKeyDown(Keys.A) && oldstate.IsKeyDown(Keys.A))
                {
                    if (jumped == false)
                        cirPos.Y -= cirSpeed * 10;
                    jumped = true;
                }
                if (padState.IsButtonDown(Buttons.A))
                {
                    if (jumped == false)
                        cirPos.Y -= cirSpeed*10;
                    jumped = true;
                }

                //** GamePad support ** //
                cirPos.X += padState.ThumbSticks.Left.X * cirSpeed;
                
                
                // ***** DIDN'T FINISH THE CHECK COLLISION ***** //
                //If player hits the platform, go up
                //CheckCollision()
                //for every time you hit a platform, increment highscore += regPt
                Start();

                //Gravity pulls object down constantly
                cirPos.Y += gravity;
                

                CheckBounds();          //Make sure object is within viewable area
            }

            oldstate = state;
            base.Update(gameTime);
        }

        private void CheckBounds()
        {
            //Checks to make sure that the ship is in bounds of the window
            // if it is past bound, set position to be at boundary

            //Prevent ship from going offscreen to the left
            if (cirPos.X < 0 + cirTexture.Width / 2)
                cirPos.X = cirTexture.Width / 2;
            //Prevent ship from going offscreen to the right
            if (cirPos.X > Window.ClientBounds.Width / 2 /*+ cirTexture.Width / 2*/)
                cirPos.X = Window.ClientBounds.Width / 2 /*+ cirTexture.Width / 2*/;
            //Prevent ship from going offscreen too low
            if (cirPos.Y < cirTexture.Height / 2)
                cirPos.Y = cirTexture.Height / 2;
            //Preven ship from going offscreen too high
            if (cirPos.Y > Window.ClientBounds.Height - cirTexture.Height / 2)
                cirPos.Y = Window.ClientBounds.Height - cirTexture.Height / 2;
            //Reset to allow jump if player touches bottom
            if (cirPos.Y == Window.ClientBounds.Height - cirTexture.Height / 2)
                jumped = false;
        }
        private void Start()
        {
            //Create the platforms
            for (int i = 0; i < 10; i++)
            {
                Components.Add(new Meteor(this, ref platTexture, cirPos));
            }
        }
        public Rectangle GetBounds()
        {
            return new Rectangle((int)cirPos.X, (int)cirPos.Y, 
                cirTexture.Width, cirTexture.Height);
        }
        private void DoGameLogic()
        {
            //check collisions
            bool hasCollision = false;
            Rectangle cirRect = GetBounds();

            foreach (GameComponent gc in Components)
            {
                if (gc is Meteor)
                {
                    hasCollision = ((Meteor)gc).checkCollision(cirRect);
                    if (hasCollision)
                    {
                        //Do code to boost up circle
                    }
                }
            }            
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //Draw the stuff
            
            //Load the menu screen
            if (pastMenu == false)
            {
                spriteBatch.Begin();
                spriteBatch.Draw(menuTexture,
                    new Vector2(0, 0), null, Color.White);
                spriteBatch.End();
            }
            //Load the game screen
            else
            {
                spriteBatch.Begin();
                //Draw the background
                spriteBatch.Draw(mainTexture,
                    new Vector2(0, 0), null, Color.LimeGreen);


                //Draw the platforms
                spriteBatch.Draw(platTexture, p
                // End draw platforms

                //Draw the circle
                spriteBatch.Draw(cirTexture, cirPos,
                            null, Color.White, 0, origin, 1, SpriteEffects.None, 1);

                //Draw the scoreboard
                spriteBatch.DrawString(gameFont, "High Score: " + highscore.ToString(),
                    new Vector2((2 * Window.ClientBounds.Width / 3), (Window.ClientBounds.Height / 3)),
                        Color.White);
                spriteBatch.End();
            }

            //Load the instructions
            if (instMenu == true)
            {
                spriteBatch.Begin();
                spriteBatch.Draw(menu2Texture,
                    new Vector2(0, 0), null, Color.White);
                spriteBatch.End();
            }

            base.Draw(gameTime);
        }
    }
}
