using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace CIS297_FinalProj
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Platf : Game1
    {
        protected Texture2D texture;            //For local call of the platform texture
        protected Rectangle platRectangle;      //Defines the boundary
        public Vector2 platPos;                 //The position of the platform
        protected int platSpeed = 4;            //Platform movement speed
        protected Random random;                //Random generator
        

        public Platf(ref Texture2D platTexture, Vector2 cirPos, int platCount)
        {
            texture = platTexture;
            platPos = new Vector2();

            //This is the boundary for the platform
            //platRectangle = new Rectangle(0, 0, platTexture.Width / 2, platTexture.Height / 2);

            //Initialize random number generator
            random = new Random(this.GetHashCode());
            PutinStartPosition(cirPos, platCount);
        }
        public void PutinStartPosition(Vector2 cirPos, int platCount)
        {
            //Place x anywhere randomly on screen
            platPos.X = random.Next(Window.ClientBounds.Width / 2 - platRectangle.Width);
            platPos.Y = cirPos.Y - 50 * platCount;// +cirPos.Y * 5 / 4;
        }
        public bool checkCollision(Rectangle rect)
        {
            Rectangle spriterect = new Rectangle((int) platPos.X, (int)platPos.Y,
                texture.Width, texture.Height);
            return spriterect.Intersects(rect);
        }
    }
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        private SpriteFont gameFont;

        //Variables
        List<Platf> pList = new List<Platf>();
        Texture2D cirTexture;
        Texture2D platTexture;
        Texture2D mainTexture;
        Texture2D animBgTexture;
        Texture2D menuTexture;
        Texture2D menu2Texture;
        Vector2 cirPos;                         //Position of the circle
        Vector2 origin;                         //Center of the circle
        Rectangle cirBound;
        KeyboardState oldstate;                 //Used to determine key press and release
        KeyboardState state;                    //Used to determine key presses
        const float cirSpeed = 10.0f;            //Speed of the circle
        const float gravity = 2.0f;            //Set gravity
        const int regPt = 25;                   //Regular points value
        const int spcPt = 100;                  //Special points value
        const int MAX_PLATF = 1;
        public int highscore = 0;               //Holds number of points
        public int platCount = 0;
        private float timeSoFar = 0.0f;         //The total running time
        bool pastMenu = false;
        bool instMenu = false;
        bool jumped = false;

        //Audio
        AudioEngine audioEngine;
        SoundBank soundBank;
        WaveBank waveBank;
        Cue EngineSound = null;


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            //Size of screen
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Services.AddService(typeof(SpriteBatch), spriteBatch);

            // Load in the assets
            cirTexture = Content.Load<Texture2D>(@"images\circle");
            platTexture = Content.Load<Texture2D>(@"images\platform");
            mainTexture = Content.Load<Texture2D>(@"images\main");
            menuTexture = Content.Load<Texture2D>(@"images\Menu");
            menu2Texture = Content.Load<Texture2D>(@"images\Instructions");
            // add animated background here
            gameFont = Content.Load<SpriteFont>(@"Font\\SpriteFont1");
            audioEngine = new AudioEngine(@"Content\Audio\audio.xgs");
            soundBank = new SoundBank(audioEngine, @"Content\Audio\Sound Bank.xsb");
            waveBank = new WaveBank(audioEngine, @"Content\Audio\Wave Bank.xwb");

            // Find the center of the circle
            origin.X = cirTexture.Width / 2;
            origin.Y = cirTexture.Height / 2;
            // Set position of the circle
            cirPos = new Vector2(Window.ClientBounds.Width / 4,
                        Window.ClientBounds.Height);
            
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            GamePadState padState = GamePad.GetState(PlayerIndex.One);      //Load up first player to controller

            // Allows the game to exit
            if ((GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed ||
                Keyboard.GetState().IsKeyDown(Keys.Escape)))
                this.Exit();

            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;   //The amount of time since last called

            timeSoFar += (float)gameTime.ElapsedGameTime.TotalSeconds;      //Continually update amount of time passed
            
            //pastMenu = true;                    //Disable moving up and down

            // Update status of keys pressed
            state = Keyboard.GetState();

            //If pressed the Enter or Start key, begin the game
            if (padState.Buttons.Start == ButtonState.Pressed ||
                state.IsKeyDown(Keys.Enter))
            {
                if (state.IsKeyDown(Keys.Enter) && !oldstate.IsKeyDown(Keys.Enter))
                {
                    pastMenu = true;
                }
                if (padState.IsButtonDown(Buttons.Start))
                {
                    pastMenu = true;
                }
            }

            //If pressed the Space or Select key, show instructions
            if (padState.Buttons.Back == ButtonState.Pressed ||
                state.IsKeyDown(Keys.Space))
            {
                if (state.IsKeyDown(Keys.Space) && !oldstate.IsKeyDown(Keys.Space))
                {
                    instMenu = true;
                }
                if (padState.IsButtonDown(Buttons.Back))
                {
                    instMenu = true;
                }
            }
            //Press B to exit instructions
            if (padState.IsButtonDown(Buttons.B) ||
                state.IsKeyDown(Keys.B))
                instMenu = false;

            if (pastMenu == true)
            {
                //** Keyboard Support **//

                //Move Left
                if (state.IsKeyDown(Keys.Left))
                    cirPos.X -= cirSpeed;
                //Move Right
                if (state.IsKeyDown(Keys.Right))
                    cirPos.X += cirSpeed;

                //Allow player to jump once
                if (state.IsKeyDown(Keys.A) && oldstate.IsKeyDown(Keys.A))
                {
                    if (jumped == false)
                        cirPos.Y -= cirSpeed * 10;
                    jumped = true;
                }
                if (padState.IsButtonDown(Buttons.A))
                {
                    if (jumped == false)
                        cirPos.Y -= cirSpeed*10;
                    jumped = true;
                }

                //** GamePad support ** //
                cirPos.X += padState.ThumbSticks.Left.X * cirSpeed;
                
                
                // ***** DIDN'T FINISH THE CHECK COLLISION ***** //
                //If player hits the platform, go up
                //CheckCollision()
                //for every time you hit a platform, increment highscore += regPt
                Start();
                DoGameLogic();

                //Gravity pulls object down constantly
                cirPos.Y += gravity;
                

                CheckBounds();          //Make sure object is within viewable area
            }

            oldstate = state;
            base.Update(gameTime);
        }

        private void CheckBounds()
        {
            //Checks to make sure that the ship is in bounds of the window
            // if it is past bound, set position to be at boundary

            //Prevent circle from going offscreen to the left
            if (cirPos.X < 0 + cirTexture.Width / 2)
                cirPos.X = cirTexture.Width / 2;
            //Prevent circle from going offscreen to the right
            if (cirPos.X > Window.ClientBounds.Width / 2 + cirTexture.Width / 2)
                cirPos.X = Window.ClientBounds.Width / 2 + cirTexture.Width / 2;
            //Prevent circle from going offscreen too high
            if (cirPos.Y < cirTexture.Height / 2)
                cirPos.Y = cirTexture.Height / 2;
            //Preven circle from going offscreen too low
            if (cirPos.Y > Window.ClientBounds.Height - cirTexture.Height / 2)
                cirPos.Y = Window.ClientBounds.Height - cirTexture.Height / 2;
            //Reset to allow jump if player touches bottom
            if (cirPos.Y == Window.ClientBounds.Height - cirTexture.Height / 2)
                if (jumped == true)
                {
                    highscore = 0;
                    jumped = false;

                    //for (int i = 0; i < platCount; i++)
                    //{
                    //    pList.Remove(pList[i]);
                    //}
                    //platCount = 0;
                    reset();
                }

            // ** Shift scene downwards not working correctly
            ////Code will be redundant against one of the above codes
            //// if circle gets above half of screen, shift the "scene"
            //// by shifting the circle and all platforms down
            //if (cirPos.Y < Window.ClientBounds.Height / 2 + cirTexture.Height / 2)
            //{
            //    cirPos.Y += Window.ClientBounds.Height / 2 - cirTexture.Height / 2;
            //    for (int i = 0; i < 10; i++)
            //    {
            //        pList[i].platPos.Y += Window.ClientBounds.Height/2 - cirTexture.Height/2;
            //    }
            //}

        }
        private void Start()
        {
            //Create the platforms
            //for (int i = 0; i < MAX_PLATF; i++)
            //{
                //Components.Add(new Meteor(this, ref platTexture, cirPos, i));
                //pList[i] = new Platf(ref platTexture, cirPos, i);
                if (platCount < 5)
                {
                    pList.Add(new Platf(ref platTexture, cirPos, platCount));
                    platCount++;
                }
            //}
            //DoGameLogic();

        }
        public Rectangle GetBounds()
        {
            return new Rectangle((int)cirPos.X+cirTexture.Width/2, (int)cirPos.Y+cirTexture.Height/2, 
                (int)cirTexture.Width, (int)cirTexture.Height);
        }
        private void DoGameLogic()
        {
            //check collisions
            bool hasCollision = false;
            Rectangle cirRect = GetBounds();

            //int counter = 0;
            //foreach (Platf gc in pList)
            //{
            //    if (gc is Platf)
            //    {
            //        hasCollision = ((Platf)gc).checkCollision(cirRect);
            //        if (hasCollision)
            //        {
            //            //Do code to boost up circle
            //            cirPos.Y -= 10;
            //            pList.Remove(pList[counter]);
            //        }
            //    }
            //    counter++;
            //}
            ;
            for (int i = 0; i < 1; i++)
            {
                //if (pList[i] is Platf)
                //{
                hasCollision = (pList[i].checkCollision(cirRect));

                if (hasCollision)
                {
                    cirPos.Y -= 100;
                    pList.Remove(pList[i]);
                    highscore += regPt;

                    platCount--;
                }
                //}
            }
        }
        public void reset()
        {
            for (int i = 0; i < platCount; i++)
            {
                if (pList[i] is Platf)
                {
                    pList.RemoveAt(i);
                    i--;
                    platCount--;
                }
            }
        }
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //Draw the stuff
            
            //Load the menu screen
            if (pastMenu == false)
            {
                spriteBatch.Begin();
                spriteBatch.Draw(menuTexture,
                    new Vector2(0, 0), null, Color.White);
                spriteBatch.End();
            }
            //Load the game screen
            else
            {
                spriteBatch.Begin();
                //Draw the background
                spriteBatch.Draw(mainTexture,
                    new Vector2(0, 0), null, Color.LimeGreen);


                //Draw the platforms
                foreach (Platf i in pList)
                {
                    spriteBatch.Draw(platTexture, i.platPos,
                                null, Color.WhiteSmoke, 0, new Vector2(0,0), 1, SpriteEffects.None, 1);
                }
                // End draw platforms

                //Draw the circle
                spriteBatch.Draw(cirTexture, cirPos,
                            null, Color.White, 0, origin, 1, SpriteEffects.None, 1);

                //Draw the scoreboard
                spriteBatch.DrawString(gameFont, "High Score: " + highscore.ToString(),
                    new Vector2((2 * Window.ClientBounds.Width / 3), (Window.ClientBounds.Height / 3)),
                        Color.White);
                spriteBatch.End();
            }

            //Load the instructions
            if (instMenu == true)
            {
                spriteBatch.Begin();
                spriteBatch.Draw(menu2Texture,
                    new Vector2(0, 0), null, Color.White);
                spriteBatch.End();
            }

            base.Draw(gameTime);
        }
    }
}
